PROCEDURE select_files_toMove
*!*	-------------------------------------------------------------------------------------
*!*	Propsito			Levantar un array con datos de archivos de una carpeta (tcSource_Folder),
*!*						segn la mscara que se pasa como parmetro (tcMask),
*!*						para ser movidos a otra carpeta (tcTarget_folder) de acuerdo a la fecha
*!*						tope que se pasa como parmetro (ttFechaHora)
*!*	-------------------------------------------------------------------------------------
*!*	UserDefineFunctions		move_Old_File() Intenta mover el archivo pasado como parmetro
*!*							ptag()			Se utiliza para generar un reporte tipo xml
*!*							ShowMeError()				Rutina que muestra un error
*!*	-------------------------------------------------------------------------------------
*!* Parameters
*!*	tcMask:				Mscara de filtro para adir. Default = "*.*"
*!*	tcSource_Folder		Carpeta de origen
*!*	tcTarget_Folder		Carepta de Destino
*!*	ttFechaHora:		Fecha y Hora de comparacin. Si se pasa solamente un valor Date,
*!*							el programa lo convierte en DateTime
*!*						Formato Date o DateTime
*!*						Si se omite el parmetro o su tipo de valor #"DT"
*!*						el programa intentar mover el archivo (lForceMode)
*!*	-------------------------------------------------------------------------------------
LPARAMETERS tcMask,tcSource_Folder,m.tcTarget_Folder,ttFechaHora

TRY 

	LOCAL LOEX AS OBJECT,;
		loResult as Object,;
		lcFileLog,;
		i,;
		lnFiles,;
		lcString,;
		LCMESSAGE
		
	LOCAL ARRAY laFiles(1)
	
	lcFileLog = JUSTPATH("")+"FilesMoved.log"
	IF !FILE(m.lcFileLog)
		STRTOFILE("",m.lcFileLog)
	ENDIF 
	
	IF VARTYPE(m.tcMask)#"C" OR EMPTY(m.tcMask)
		tcMask = "*"
	ENDIF 
	DO WHILE .t.
		IF VARTYPE(m.tcSource_folder)#"C" OR EMPTY(m.tcSource_Folder)
			lcMessage = "Se omiti el parmetro Source_Folder"
			EXIT
		ENDIF 
		IF !DIRECTORY(m.tcSource_Folder)
			lcMessage = "Carpeta origen "+m.tcSource_folder+" no existe"
			EXIT
		ENDIF 

		IF VARTYPE(m.tcTarget_Folder)#"C" OR EMPTY(m.tcTarget_Folder)
			lcMessage = "Parmetro Target_Folder incorrecto"
			EXIT
		ENDIF 
		IF !Directory(m.tcTarget_Folder)
			lcMessage = "Carpeta destino "+m.tcTarget_Folder+" Inexistente"
			EXIT 
		ENDIF 

		lcString = ptag("files_moved,timestamp,"+TTOC(DATETIME(),3),null,0,1);
			+ Ptag("parameters",null,1,1);
			+ Ptag("mask",m.tcMask,2,1);
			+ Ptag("source_folder",m.tcSource_Folder,2,1);
			+ Ptag("target_folder",m.tcTarget_Folder,2,1);
			+ Ptag("fecha_hora",m.ttFechaHora,2,1);
			+ Ptag("/parameters",null,1,1)
			
		
		lnFiles = ADIR(laFiles,ADDBS(m.tcSource_Folder)+m.tcMask,"",1)
		lcString = m.lcSTring ;
			+ Ptag("files_detected",m.lnFiles,1,1);
			+ Ptag("report",null,1,1)
			
		FOR i = 1 TO m.lnFiles
			loREsult = move_Old_File(ADDBS(m.tcSource_Folder)+laFiles[i,1],;
					m.tcTarget_Folder,;
					m.ttFechaHora)
			
			lcString = m.lcSTring ;
				+ Ptag("file,name,"+loREsult.Source_File,null,2,0);
				+ Ptag("forced",loREsult.ForceMode);
				+ Ptag("doit",loREsult.lDoit);
				+ Ptag("moved",loREsult.lMoved,0,1)
				
		NEXT 
		lcString = m.lcString ;
			+ Ptag("/report",null,1,1);
			+ Ptag("/files_moved",null,0,1)
		
		STRTOFILE(m.lcString + CHR(13) + FILETOSTR(m.lcFileLog),m.lcFilelog)
		
		EXIT
	ENDDO 
	
		
CATCH TO LOEX
	LOEX.USERVALUE = PROGRAM()
	ShowMeError(LOEX)

FINALLY
	IF !EMPTY(LCMESSAGE)
		MESSAGEBOX(M.LCMESSAGE,0,PROPER(PROGRAM()))
	ENDIF

ENDTRY


PROCEDURE move_Old_File
*!*	--------------------------------------------------------------------------------------
*!*	Move_Old_File(lcPath_FileName,lcTarget_Folder,ltFechaHora)
*!*	--------------------------------------------------------------------------------------
*!*	tcPath_fileName:	Ruta, nombre y extensin del archivo a mover
*!*	tcTarget_Folder:	Ruta y carpeta de destino del archivo a mover
*!*	ttFechaHora:		Fecha y Hora de comparacin. Si se pasa solamente un valor Date,
*!*							el programa lo convierte en DateTime
*!*						Formato Date o DateTime
*!*						Si se omite el parmetro o su tipo de valor #"DT"
*!*						el programa intentar mover el archivo (lForceMode)
*!*	--------------------------------------------------------------------------------------					
*!*	Retorna				Un objeto con las siguientes propiedades
*!*						.Source_file		&& parmetro tcPath_FileName
*!*						.Target_folder		&& parmetro tcTarget_Folder
*!*						.datetime_param		&& parmetro ttFechaHora o su modificacin
*!*						.ForceMode			&& Si se utiliz lForceMode
*!*						.lDoit				&& .t. cuando corresponda mover el archivo
*!*						.lMoved				&& Si el archivo fu movido a su destino
*!* --------------------------------------------------------------------------------------
*!* Warning				Si el archivo existe en destino, el programa intentar borrarlo.
*!*						Si por alguna razn no puede borrarlo, tampoco mover el archivo
*!*						de origen.
*!*	--------------------------------------------------------------------------------------
*!*	UserDefinedFunctions	Date_Time_to_DateTime()		Convierte un valor date en DateTime
*!*							ShowMeError()				Rutina que muestra un error
*!*	--------------------------------------------------------------------------------------
LPARAMETERS tcPath_FileName, tcTarget_Folder,ttFechaHora
TRY 


	LOCAL loex as Exception,;
		loex_fsoMove as Exception,;
		loex_fsoDele as Exception,;
		LoFso AS OBJECT,;
		loFile as Object,;
		loDevol as Object,;
		LCMESSAGE,;
		ltDateLastModified 
	
	loDevol = NEWOBJECT("Empty")
	ADDPROPERTY(loDevol,"source_file",m.tcPath_FileName)
	ADDPROPERTY(loDevol,"target_folder",m.tcTarget_Folder)
	ADDPROPERTY(loDevol,"datetime_param",m.ttFechaHora)
	ADDPROPERTY(loDevol,"ForceMode",.f.)
	ADDPROPERTY(loDevol,"lDoit",.f.)
	ADDPROPERTY(loDevol,"lMoved",.f.)
	
	loFso = NEWOBJECT("Scripting.FileSystemObject")
	
	DO CASE 
		CASE VARTYPE(m.ttFechaHora) = "T"
			* No hace nada
		CASE VARTYPE(m.ttFechaHora) = "D"
			ttFechaHora = Date_Time_to_DateTime(m.ttFechaHora)
		OTHERWISE 
			loDevol.ForceMode = .t.
	ENDCASE 
	
	DO WHILE .t.
		IF VARTYPE(m.tcPath_FileName)#"C" OR EMPTY(m.tcPath_FileName)
			lcMessage = "Parmetro Path_FileName incorrecto"
			EXIT
		ENDIF 
		IF !loFso.FileExists(m.tcPath_FileName)
			lcMessage = "El archivo "+m.tcPath_FileName+" no existe"
			EXIT
		ENDIF 
		
		IF VARTYPE(m.tcTarget_Folder)#"C" OR EMPTY(m.tcTarget_Folder)
			lcMessage = "Parmetro Target_Folder incorrecto"
			EXIT
		ENDIF 
		IF !loFso.FolderExists(m.tcTarget_Folder)
			lcMessage = "Carpeta "+m.tcTarget_Folder+" Inexistente"
			EXIT 
		ENDIF 
		
		IF loDevol.ForceMode
			loDevol.lDoit = .t.
		ELSE 
			loFile = loFso.GetFile(m.tcPath_FileName)
			ltDateLastModified = loFile.DateLastModified()
			loDevol.lDoit = m.ltDateLastModified < m.ttFechaHora
		ENDIF 

		lcTarget_File = ADDBS(m.tcTarget_Folder);
			+ JUSTFNAME(m.tcPath_FileName)
		
		IF loDevol.lDoit
			IF loFso.FileExists(m.lcTarget_File)
				TRY 
					loFso.DeleteFile(m.lcTarget_File)
				CATCH TO loex_FsoDele
					loex_FsoDele.UserValue = "No pudo borrarse "+m.lcTarget_File
					THROW
				FINALLY
				ENDTRY 
			ENDIF 
		
		
			TRY 
				loFso.MoveFile(m.tcPath_FileName,ADDBS(m.tcTarget_Folder))
			CATCH TO loex_fsoMove
				loex_fsoMove.UserValue = "Error al intentar mover "+m.tcPath_filename
				THROW 
			FINALLY
			
			ENDTRY 
		
		ENDIF 

			
		WITH loDevol
			.datetime_Param = m.ttFechaHora
			.lMoved = loDevol.lDoit and loFso.FileExists(m.lcTarget_File)
		ENDWITH 

	EXIT
	ENDDO 

		
CATCH TO LOEX
	LOEX.USERVALUE = PROGRAM() ;
		+ CHR(13) ;
		+ IIF(VARTYPE(loex_FsoDele)="O",loex_FsoDele.UserValue+CHR(13),"");
		+ IIF(VARTYPE(loex_FsoMove)="O",loex_FsoMove.UserValue,"")
		
	ShowMeError(LOEX)

FINALLY
	loFile = null
	loFso = null
	
	IF !EMPTY(LCMESSAGE)
		MESSAGEBOX(M.LCMESSAGE,0,PROPER(PROGRAM()))
	ENDIF


ENDTRY
RETURN loDevol
ENDPROC

PROCEDURE Date_Time_to_DateTime
* ----------------------------------------
* Convertir un valor Date (tdFecha) 
* y un valor Time (tcTime) 
* en un DateTime (Valor Retornado)
* ----------------------------------------
LPARAMETERS tdFecha,tcHora,tnHours
TRY 

	LOCAL lnSep,;
		lnAdd,;
		ltReturn,;
		lcString,;
		loex as Exception

	LOCAL ARRAY laHora(1)

	
	ltReturn = {//::}
	
	* Fecha
	IF VARTYPE(m.tdFecha)#"D"
		tdFEcha = {}
	ENDIF 
		
	* Hora
	IF VARTYPE(m.tcHora)#"C"
		tcHora = "00:00:00"
	ENDIF 
	tcHora = UPPER(m.tcHora)
	lnAdd=IIF(AT("PM",tcHora)>0,12,0)
	
	* hours
	IF VARTYPE(tnHours)#"N" OR !BETWEEN(m.tnHours,12,24)
		tnHours = 24
	ENDIF 

	SET HOURS TO tnhours

	IF !EMPTY(m.tdFecha)
		lnSep = ALINES(laHora,CHRTRAN(m.tcHora,CHRTRAN(m.tcHora,"1234567890:",""),""),1,":")
		IF m.lnSep > 0
			IF m.lnAdd = 12
				laHora[1] = TRANSFORM(VAL(laHora[1])+12)
			ENDIF 
			IF VAL(laHora[1])=24
				tdFecha = m.tdFEcha + 1
				laHora[1] = "00"
			ENDIF 		
		ENDIF 
		ltReturn = DATETIME(YEAR(m.tdFecha),;
			MONTH(m.tdFecha),;
			DAY(m.tdFecha),;
			IIF(m.lnSep>0,VAL(laHora[1])+lnAdd,0),;
			IIF(m.lnSep>1,VAL(laHora[2]),0),;
			IIF(m.lnSep>2,VAL(laHora[3]),0))
			
	ENDIF
CATCH TO loex
	lcSTring = "tdFecha="+IIF(VARTYPE(m.tdFecha)="C",tdFecha,TRANSFORM(tdFecha)) + CHR(13);
		+ "tcHora="+IIF(VARTYPE(m.tcHora)="C",tcHora,TRANSFORM(m.tcHora)) + CHR(13) ;
		+ "lnSep="+TRANSFORM(m.lnSep) + CHR(13)
		
	loex.UserValue = PROGRAM()+CHR(13);
		+ m.lcString
		
	ShowMeError(loex)

FINALLY

ENDTRY
RETURN m.ltReturn
ENDPROC 

****************************************************************
PROCEDURE Ptag
*!* ------------------------------------------------------------
*!*	Ptag 	(Crear una Tag para xml )
*!* ------------------------------------------------------------
*!*	tcTag: 		Nombre de etiqueta
*!*	txValue:		Valor de la etiqueta (Types "C,D,T,L,I,N")
*!*	tnLevel:		Cantidad de Tabs que anteceden la etiqueta
*!*	tn_Add_CR:		Si devuelve un retorno de carro:
*!*					.f. No
*!*					.t. 1 retorno
*!*					0 	No
*!*					1	1 retorno
*!*					2	2 retornos
*!* ------------------------------------------------------------
*!*	For xml Attribute Value  tcTag="Class,name,Pirincho"  ->  [ Class name="Pirincho" ]				
*!* ------------------------------------------------------------
*!* tnFlag_Char_EofLine
*!*		.f.,.t.,0		CHR(13) + CHR(10)
*!*		1				CHR(13)
*!*		2				CHR(10)
*!* Cualquier valor mayor que 2, funciona = 0
*!* ------------------------------------------------------------
*!*
*!*	tnFlag
*!* ---------------------------
*!*	0 - No hace nada (Default)
*!*	1 - Avoid format number or Format Date or Format Datetime
*!*	3 - For Datetime Use Xml Format 
*!*	
*!* ------------------------------------------------------------

LPARAMETERS tcTag,txValue,tnLevel,tn_Add_CR ,tnFlag_Char_EofLine ,tnFlag

tnLevel=EVL(tnLevel,0)
tnFlag = EVL(tnflag,0)
				
TRY 
	LOCAL i_,;
		lcChar_EofLine,;
		lcmask,;
		lcReturn,;
		lcTag_Passed,;
		lcValue,;
		lcVartype,;
		lnCarriage_Return,;
		lnCommas,;
		lnlines,;
		loex AS Exception


		
	lcTag_Passed = tcTag	

	IF VARTYPE(m.tnFlag_Char_EofLine)#"N"
		tnFlag_Char_EofLine = 0
	ENDIF 
	lcChar_EofLine = ICASE(m.tnFlag_Char_EofLine = 1, CHR(13),;
					tnFlag_Char_EofLine = 2 , CHR(10),;
					CHR(13)+CHR(10))
					
	LOCAL ARRAY laLines(1)

	tcTag=ALLTRIM(tcTag)

	*<xml_Attribute>
	lnCommas = OCCURS(CHR(44),tcTag)
	IF lnCommas > 1
		lcValue = ""
		lnLines=ALINES(laLines , tcTag , 2 ,CHR(44))
		FOR i_=3 TO lnLines
			lcValue = m.lcValue + lalines[i_]
		NEXT
		tcTag = laLines[1]+" "+ALLTRIM(lalines[2])+CHR(61) ;
			+IIF(EMPTY(m.lcValue),CHR(34)+CHR(34),CHR(34)+m.lcValue+CHR(34))
		
	ENDIF
	*</xml_Attribute>

	lnCarriage_Return = ICASE(VARTYPE(tn_Add_CR)="L",	IIF(tn_Add_CR,1,0) , ;
				VARTYPE(tn_Add_CR)="N" , tn_Add_CR , ;
				0 )
				
	IF ISNULL(txValue)
		lcREturn=+"<"+m.tcTag+">"
	ELSE
		lcVartype=VARTYPE(m.txValue)
		DO CASE 
			CASE m.lcVartype="N"
				lcValue = MyTRANSFORM(m.txValue)	
			CASE m.lcVartype="D"
				lcValue = IIF(m.tnFlag=1,DTOS(m.txValue),TRANSFORM(m.txValue))
				
			CASE m.lcVartype="T"
				lcValue = IIF(INLIST(m.tnFlag,1,3),TTOC(m.txValue,m.tnFlag),TRANSFORM(m.txValue))
			
			CASE m.lcVartype = "O"
				lcValue = "(Object)"
			OTHERWISE
			
				lcValue=ICASE(m.lcVartype$'IL',TRANSFORM(m.txValue),;
					lcVartype="C",ALLTRIM(m.txValue),;
					m.txValue)
		ENDCASE 
		
		lcReturn = IIF(lnCommas=0,;
				"<"+tcTag+">" + m.lcValue +"</" + m.tcTag +">", ;
				"<"+tcTag+" />")
				
	ENDIF
CATCH TO loex
	loex.UserValue = PROGRAM() + CHR(13);
		+"lcTag_Passed= "+lcTag_Passed + CHR(13);
		+"Value = "+IIF(VARTYPE(m.txValue)="C",m.txValue,TRANSFORM(m.txValue))
	ShowMeError(loex)
FINALLY
ENDTRY 

RETURN REPLICATE(CHR(9),m.tnLevel) ;
		+ m.lcReturn ;
		+ REPLICATE(m.lcChar_EofLine,m.lnCarriage_Return )
		
ENDPROC


*----------------------------------------------------
PROCEDURE ShowMeError
LPARAMETERS toExcep,tlNotShow,tcCaption
*-----------------------------------------------------
* Mostrar errores de Try/Catch/Endtry
*-----------------------------------------------------
tcCaption=EVL(tcCaption,"Mensaje del Sistema")
SET HOURS TO 24
LOCAL lcMens


lcMens="Fecha "+TRANSFORM(DATETIME());
	+ Chr(13)+"Mensaje: "+toExcep.message;
	+ Chr(13)+"ErrorNo: "+TRANSFORM(toExcep.Errorno);
	+ Chr(13)+"Llamada: "+toExcep.Uservalue
IF _vfp.StartMode=0
	lcMens=lcMens+ Chr(13)+"Linea "+TRANSFORM(toExcep.lineno)
ENDIF

STRTOFILE(lcMens+ Chr(13)+REPLICATE("=",60)+ Chr(13),"Errors.log",1)

lcMens="Se ha producido un error:"+ Chr(13)+lcMens
IF !tlNotShow
	MESSAGEBOX(m.lcMens,0,m.tcCaption)
ENDIF

ENDPROC
